@@ -35,6 +35,22 @@
 (require 'dabbrev)
 
 
+;; ---------------------------------------------------------------------------
+;; Compatibility
+
+(when (and (version< emacs-version "31.1") (not (and (fboundp 'incf) (fboundp 'decf))))
+  (defmacro incf (place &optional delta)
+    "Increment PLACE by DELTA or 1."
+    (declare (debug (gv-place &optional form)))
+    (gv-letplace (getter setter) place
+      (funcall setter `(+ ,getter ,(or delta 1)))))
+  (defmacro decf (place &optional delta)
+    "Decrement PLACE by DELTA or 1."
+    (declare (debug (gv-place &optional form)))
+    (gv-letplace (getter setter) place
+      (funcall setter `(- ,getter ,(or delta 1))))))
+
+
 ;; ---------------------------------------------------------------------------
 ;; Custom Variables
 
@@ -71,6 +87,10 @@
 (defvar-local recomplete--alist nil
   "Internal properties for repeated `recomplete' calls.")
 
+
+;; ---------------------------------------------------------------------------
+;; Implementation: Utilities
+
 (defmacro recomplete--with-advice (advice &rest body)
   "Execute BODY with ADVICE temporarily enabled.
 
@@ -462,8 +482,8 @@ Return the region replaced."
               (setq len-test i))))))
       (unless (zerop i)
         (setq i-end (- len i))
-        (setq len (- len i))
-        (setq end (- end i))
+        (decf len i)
+        (decf end i)
         (setq i-end-ofs i)))
 
     ;; Check for skip start.
@@ -472,12 +492,12 @@ Return the region replaced."
         (while (< i len-test)
           (cond
            ((eq (aref str i) (char-after (+ beg i)))
-            (setq i (1+ i)))
+            (incf i))
            (t ; Break.
             (setq len-test i)))))
       (unless (zerop i)
         (setq i-beg i)
-        (setq beg (+ beg i))))
+        (incf beg i)))
 
     (when (or i-beg i-end)
       (setq str (substring str (or i-beg 0) (or i-end len))))
@@ -644,7 +664,7 @@ step onto the next item)."
                                           (format "[%s]" (propertize iter-word 'face 'match)))
                                          (t
                                           (format " %s " iter-word)))
-                                   (setq iter-index (1+ iter-index))))
+                                   (incf iter-index)))
                                result-choices))
                             "")))
 
